// Copyright 2021 Matrix Origin
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package ops

import (
	"context"
	"matrixone/pkg/vm/engine/aoe/storage/worker/base"
	"sync"
	"time"
)

type heartbeater struct {
	handle   base.IHBHandle
	interval time.Duration
	ctx      context.Context
	cancel   context.CancelFunc
	wg       *sync.WaitGroup
}

func NewHeartBeater(interval time.Duration, handle base.IHBHandle) base.IHeartbeater {
	c := &heartbeater{
		interval: interval,
		handle:   handle,
	}
	c.ctx, c.cancel = context.WithCancel(context.Background())
	return c
}

func (c *heartbeater) Start() {
	c.wg = &sync.WaitGroup{}
	c.wg.Add(1)
	go func() {
		defer c.wg.Done()
		ticker := time.NewTicker(c.interval)
		for {
			select {
			case <-c.ctx.Done():
				ticker.Stop()
				c.handle.OnStopped()
				return
			case <-ticker.C:
				c.handle.OnExec()
			}
		}
	}()
}

func (c *heartbeater) Stop() {
	c.cancel()
	c.wg.Wait()
}
